#include <iostream>

void spfilter (std::string &exp) {
    unsigned int n;
    while (true) {
	n = exp.find (' ');
	if (n < exp.size ())
	    exp.erase (n, 1);
	else
	    break;
    }
    while (true) {
	n = exp.find ('\t');
	if (n < exp.size ())
	    exp.erase (n, 1);
	else
	    break;
    }
    while (true) {
	n = exp.find ('\0');
	if (n < exp.size ())
	    exp.erase (n, 1);
	else
	    break;
    }
}

bool brcheck (const std::string exp) {
    int openbr = 0;
    int closebr = 0;
    bool err = false;
    for (unsigned int i = 0; i < exp.size (); i++) {
	if (exp.at (i) == '(') {
	    openbr++;
	}
	else if (exp.at (i) == ')') {
	    closebr++;
	    if (closebr > openbr)
		err = true;
	}
	if (err) {
	    std::cout << "Syntax error! Invalid bracket." << '\n';
	    std::cout << exp << '\n';
	    for (unsigned int j = 0; j < i; j++)
		std::cout << ' ';
	    std::cout << '^' << '\n';
	    break;
	}
    }
    if (! err && openbr != closebr) {
	std::cout << "Syntax error! Invalid brackets count." << '\n';
	err = true;
    }
    return (! err);
}

bool syntaxcheck (const std::string exp) {
    enum state { ststart, stclbr, stnum, stop, sterr = -1 };
    enum digitst { beforep, afterp };
    state st = ststart;
    digitst digst = beforep;
    bool err = false;
    for (unsigned int i = 0; i < exp.size (); i++) {
	switch (st) {
	    case ststart:
		if ( ! isdigit (exp.at (i)) &&
		     exp.at (i) != '-'      &&
		     exp.at (i) != '+'      &&
		     exp.at (i) != '('
		   ) {
		    st = sterr;
		}
		else if (isdigit (exp.at (i))) {
		    st = stnum;
		    digst = beforep;
		}
		else if (exp.at (i) == '-' || exp.at (i) == '+') {
		    st = stop;
		}
		else if (exp.at (i) == '(') {
		    digst = beforep;
		}
		break;
	    case stnum:
		if (isdigit (exp.at (i))) { }
		else if (exp.at (i) == '.') {
		    switch (digst) {
			case beforep:
			    digst = afterp;
			    break;
			case afterp:
			    st = sterr;
			    break;
		    }
		}
		else if ( exp.at (i) == '+' ||
			  exp.at (i) == '-' ||
			  exp.at (i) == '*' ||
			  exp.at (i) == '/'
		        ) {
		    st = stop;
		}
		else if (exp.at (i) == ')') {
		    st = stclbr;
		}
		else {
		    st = sterr;
		}
		break;
	    case stop:
		if (! isdigit (exp.at (i)) && exp.at (i) != '(') {
		    st = sterr;
		}
		else if (exp.at (i) == '(') {
		    st = ststart;
		}
		else {
		    st = stnum;
		    digst = beforep;
		}
		break;
	    case stclbr:
		if ( exp.at (i) == '+' ||
		     exp.at (i) == '-' ||
		     exp.at (i) == '*' ||
		     exp.at (i) == '/'
		   ) {
		    st = stop;
		}
		else if (exp.at (i) == ')') { }
		else {
		    st = sterr;
		}
		break;
	    case sterr:
	        break;
	    default:
		st = sterr;
		break;
	}
	if (st == sterr) {
	    err = true;
	    std::cout << "Syntax error! Invalid expression." << '\n';
	    std::cout << exp << '\n';
	    for (unsigned int j = 0; j < i; j++)
		std::cout << ' ';
	    std::cout << '^' << '\n';
	    break;
	}
    }
    return (! err);
}

void unaryrep (std::string &exp) {
    for (unsigned int n = 0; n < exp.size (); n++) {
	if (n == 0 && exp.at (n) == '-') {
	    exp.replace (n, 1, "_");
	}
	else if (n == 0 && exp.at (n) == '+') {
	    exp.replace (n, 1, "~");
	}
	else if (n > 0 && exp.at (n - 1) == '(' && exp.at (n) == '-') {
	    exp.replace (n, 1, "_");
	}
	else if (n > 0 && exp.at (n - 1) == '(' && exp.at (n) == '+') {
	    exp.replace (n, 1, "~");
	}
    }
}

double exptof (const std::string exp) {
    double val, power;
    int sign;
    int i = 0;

    sign = (exp.at (i) == '-') ? -1 : 1;
    if (exp.at (i) == '+' || exp.at (i) == '-')
	i++;
    for (val = 0.0; isdigit (exp.at (i)); i++)
	val = 10.0 * val + (exp.at (i) - '0');
    if (exp.at (i) == '.')
	i++;
    for (power = 1.0; isdigit (exp.at (i)); i++) {
	val = 10.0 * val + (exp.at (i) - '0');
	power *= 10.0;
    }
    return  sign * val / power;
}

std::string exptopol (const std::string exp) {
    std::string stackop = "";
    std::string polexp = "";

    for (unsigned int i = 0; i < exp.size (); i++) {
	switch (exp.at (i)) {
	    case '0': case '1': case '2': case '3': case '4':
	    case '5': case '6': case '7': case '8': case '9':
	    case '.':
		polexp += exp.at (i);
		break;
	    case '(':
		stackop += exp.at (i);
		polexp += ' ';
		break;
	    case ')':
		while (stackop.at (stackop.size () - 1) != '(') {
		    polexp += stackop.at (stackop.size () - 1);
		    stackop.erase (stackop.size () - 1, 1);
		}
		stackop.erase (stackop.size () - 1, 1);
		polexp += ' ';
		break;
	    case '+': case '-':
		if (stackop.size () == 0) {
		    stackop += exp.at (i);
		}
		else {
		    while (stackop.size () > 0 && stackop.at (stackop.size () - 1) != '(') {
			polexp += stackop.at (stackop.size () - 1);
			stackop.erase (stackop.size () - 1, 1);
		    }
		    stackop += exp.at (i);
		}
		polexp += ' ';
		break;
	    case '*': case '/':
		if (stackop.size () == 0) {
		    stackop += exp.at (i);
		}
		else {
		    while (stackop.size () > 0 && stackop.at (stackop.size () - 1) != '(' &&
			   (stackop.at (stackop.size () - 1) == '*' ||
			    stackop.at (stackop.size () - 1) == '/' ||
			    stackop.at (stackop.size () - 1) == '_' ||
			    stackop.at (stackop.size () - 1) == '~')
		    ) {
			polexp += stackop.at (stackop.size () - 1);
			stackop.erase (stackop.size () - 1, 1);
		    }
		    stackop += exp.at (i);
		}
		polexp += ' ';
		break;
	    case '_': case '~':
		if (stackop.size () == 0) {
		    stackop += exp.at (i);
		}
		else {
		    while (stackop.size () > 0 && stackop.at (stackop.size () - 1) != '(' &&
			   (stackop.at (stackop.size () - 1) == '_' ||
			    stackop.at (stackop.size () - 1) == '~')
		    ) {
			polexp += stackop.at (stackop.size () - 1);
			stackop.erase (stackop.size () - 1, 1);
		    }
		    stackop += exp.at (i);
		}
		polexp += ' ';
		break;
	}
    }

    while (stackop.size () > 0) {
	polexp += stackop.at (stackop.size () - 1);
	stackop.erase (stackop.size () - 1, 1);
    }

    return polexp + '\0';
}
