#include <iostream>
#include <vector>
#include "calc.h"

struct variable {
    std::string  name;
    double       val;
    std::string  txtexp;
    bool         valflag;
};

std::vector<struct variable> vars;

struct variable setvar (std::string vname, double vval, std::string vexp, bool vflag) {
    struct variable tmpvar;

    tmpvar.name    = vname;
    tmpvar.val     = vval;
    tmpvar.txtexp  = vexp;
    tmpvar.valflag = vflag;

    return tmpvar;
}

int varsearch (const std::string varname) {
    int result = -1;

    for (unsigned int i = 0; i < vars.size (); i++) {
	if (! varname.compare (vars.at (i).name)) {
	    result = i;
	    break;
	}
    }

    return result; 
}

bool substitution (std::string &vexp) {
    bool result = false;
    int startch = -1;
    int endch = -1;
    std::string subs;
    int varindex;
    for (unsigned int i = 0; i < vexp.size (); i++) {
	if (! result && 
	    ((vexp.at (i) >= 'a' && vexp.at (i) <= 'z') || 
	     (vexp.at (i) >= 'A' && vexp.at (i) <= 'Z')   ) ||
	    vexp.at (i) == '_')
	{
	    endch = i;
	    startch = i;
	    result = true;
	}
	else if ( result && ! (
		  vexp.at (i) == '+' ||
		  vexp.at (i) == '-' ||
		  vexp.at (i) == '*' ||
		  vexp.at (i) == '/' ||
		  vexp.at (i) == '(' ||
		  vexp.at (i) == ')' ||
		  i == vexp.size () -1 )
		) {
	    endch = i;
	}
	else if (result) {
	    break;
	}
    }
    if (result) {
	subs = vexp.substr (startch, endch - startch + 1);
	if ((varindex = varsearch (subs)) > -1) {
	    vexp.replace (startch, endch - startch + 1, '(' + vars.at (varindex).txtexp + ')');
	}
	else {
	    std::cout << "Error !! Variable: " << subs << " not found." << '\n';
	    result = false;
	}
    }
    std::cout << result << '\n';
    return result;
}

bool langmain (std::string &langexp) {
    bool mresult = false;
    bool setflag = false;
    std::string leftval = "";
    std::string rightval = "+0.0";
    int varindex;
    bool vvalflag = true;
    
    for (unsigned int i = 0; i < langexp.size (); i++) {
	if (langexp.at (i) == '=') {
	    setflag = true;
	    break;
	}
    }
    if (setflag) {
	if (! langexp.substr (0, 4).compare ("var ")) {		// var name=expr
	    langexp.erase (0, 4);
	    spfilter (langexp);
	    unsigned int n = 1;
	    while (langexp.at (n) != '=') { n++; }
	    leftval = langexp.substr (0, n);
	    rightval = langexp.substr (n + 1, langexp.size () - 1);
	    if ((varindex = varsearch (leftval)) > -1) {
		std::cout << "Error !! Variable already daclared: " << leftval << '\n';
	    }
	    else {
		for (unsigned int i = 0; i < rightval.size (); i++) {
		    if (! isdigit (rightval.at (i)) && rightval.at (i) != '.') {
			vvalflag = false;
			break;
		    }
		}
		if (vvalflag) {			// var name=number
		    rightval += '\0';
		    vars.push_back (setvar (leftval, exptof (rightval), rightval, true));
		}
		else {				// var name=expr
		    vars.push_back (setvar (leftval, 0.0, rightval, false));
		}
		std::cout << " name: "    << vars.back ().name 
	          << " val: "     << vars.back ().val 
		  << " txtexp: "  << vars.back ().txtexp 
		  << " valflag: " << vars.back ().valflag << '\n';
	    }
	}
	else {			// name=expr
	    spfilter (langexp);
	    unsigned int n = 1;
	    while (langexp.at (n) != '=') { n++; }
	    leftval = langexp.substr (0, n);
	    rightval = langexp.substr (n + 1, langexp.size () - 1);
//	    std::cout << "rightval: " << rightval << '\n';
//	    std::cout << "leftval: "  << leftval  << '\n';
	    if ((varindex = varsearch (leftval)) > -1) {
		for (unsigned int i = 0; i < rightval.size (); i++) {
		    if (! isdigit (rightval.at (i)) && rightval.at (i) != '.') {
			vvalflag = false;
			break;
		    }
		}
		if (vvalflag) {			// name=number
		    rightval += '\0';
		    vars.at (varindex) = setvar (leftval, exptof (rightval), rightval, true);
		}
		else {				// name=expr
		    vars.at (varindex) = setvar (leftval, 0.0, rightval, false);
		}
		std::cout << " name: "    << vars.at (varindex).name 
	          << " val: "     << vars.at (varindex).val 
		  << " txtexp: "  << vars.at (varindex).txtexp 
		  << " valflag: " << vars.at (varindex).valflag << '\n';
	    }
	    else {
		std::cout << "Error !! Variable not daclared: " << leftval << '\n';
	    }
	}
    }
    else {
	if (! langexp.substr (0, 4).compare ("var ")) {		// var name
	    langexp.erase (0, 4);
	    spfilter (langexp);
	    if ((varindex = varsearch (langexp)) > -1) {
		std::cout << "Error !! Variable already daclared: " << langexp << '\n';
	    }
	    else {
		vars.push_back (setvar (langexp, 0.0, rightval, false));
		std::cout << " name: "    << vars.back ().name 
	          << " val: "     << vars.back ().val 
		  << " txtexp: "  << vars.back ().txtexp 
		  << " valflag: " << vars.back ().valflag << '\n';
	    }
	}
	else {			// name || expr
	    spfilter (langexp);
	    if ((varindex = varsearch (langexp)) > -1) {	// name
		if (vars.at (varindex).valflag) {
		    std::cout << "Result: " << vars.at (varindex).val << '\n';
		}
		else {
		    langexp = vars.at (varindex).txtexp;
		    while (substitution (langexp)) { }
		    mresult = true;
		}
	    }
	    else {						// expr
		spfilter (langexp);
		if (langexp.compare ("")) {
		    while (substitution (langexp)) { }
		    mresult = true;
		}
	    }
	}
    }

    return mresult;
}
